{ config, pkgs, ... }:
{
  # [HARDWARE]
  imports =
    [ # Include the results of the hardware scan.
      ./hardware-configuration.nix
    ];
  hardware.opengl = {
    enable = true;
    driSupport = true;
    driSupport32Bit = true;
  };

  # [BOOT]
  boot.loader.systemd-boot.enable = true;
  boot.loader.efi.canTouchEfiVariables = true;
  boot.swraid.enable = true;
  boot.swraid.mdadmConf = ''
    ARRAY /dev/md/deppyPC:0 level=raid0 num-devices=2 metadata=1.2 name=deppyPC:0 UUID=ba6fef6e:972796a3:a331c87a:2d973f91
      devices=/dev/sda1,/dev/sdb1
  '';

  # [AUDIO]
  environment.etc."pipewire/media-session.d/alsa-monitor.con".text = ''
    enabled = true

    rules = {
      {
        matches = [
          {
            node.name = "alsa_output.usb-SMSL_SMSL_USB_AUDIO-00.analog-stereo"
          }
        ]
     
        actions = {
          update-props = {
            audio.format = "S32_LE"
          }
        }
      },
    }
  '';

  environment.etc."pipewire/pipewire.conf".text = builtins.readFile ./pipewire.conf;

  
  # [DESKTOP / XORG / WAYLAND]
  services.xserver = {
    desktopManager.plasma5.enable = true;
    displayManager.startx.enable = true;
  };
  

  # [NETWORKING]
  networking.hostName = "computational";
  networking.networkmanager.enable = true;

  system.stateVersion = "23.11";
}
